import algds.IOUtils;
/**
 * AlgDs WS99 Aufgabe 11.
 *
 * Bestimmt das kleinste gemeinsame Vielfache zweier angegebener
 * Zahlen und gibt es aus.
 * Die geforderte applikative Version ist kgV1. Die dazugehoerige
 * imperative Version ist kgV0.
 * Die Versionen kgV2 und kgV3 kommen (fast) ohne Divisionen aus,
 * sind aber nicht gefordert.
 *
 * @author Christian Semrau, 4.11.1999, 24.12.1999<br>
 * <a href="mailto:Christian.Semrau@student.uni-magdeburg.de">
 * Christian.Semrau@student.uni-magdeburg.de</a>
 */
public class ChS_Aufg11 {

static int loops;


/**
 * Imperative Basisversion.
 *
 * loops speichert die Anzahl der Rechenschritte. Die Zuweisungen
 * an loops gehoeren nicht zum eigentlichen Algorithmus.
 */
static int kgV0(int a, int b) {
  if (a < b) {int i = a; a = b; b = i;};
	// wir bilden die Vielfachen der groesseren Zahl
  int k = a;  // k ist die Zwischensumme
  loops = 1;
  while (k % b != 0) {k += a; loops++;};
  return(k);
}

/**
 * Applikativer Algorithmus, der im Prinzip genauso arbeitet wie kgV0.
 */
static int kgV1(int a, int b) {
  loops = 0;
  return( (a < b) ? kgV1_1(b, a, b) : kgV1_1(a, b, a));
  // Aufruf der Hilfsfunktion, die groessere Zahl steht vorn.
}

/**
 * Hilfsfunktion von kgV1.
 */
static int kgV1_1(int a, int b, int k) {
  loops++;
  return( (k % b == 0) ? k : kgV1_1(a, b, k+a) );
  // k ist die Zwischensumme und irgendwann die Endsumme
}

/**
 * Applikative Version, die (fast) ohne Modulo auskommt.
 *
 * kgV2 und kgV2_1 bilden einen applikativen Algorithmus,
 * dessen Hilfsfunktion viel komplizierter aussieht, aber dafuer
 * ohne Modulo auskommt.
 * kgV2 braucht genausoviele Schritte wie kgV1, die Einzelschritte sind
 * aber ein bisschen schneller als die von kgV1 (sollten sie :-).
 */
static int kgV2(int a, int b) {
  loops = 0;
  // Aufruf der Hilfsfunktion, die groessere Zahl steht vorn
  return( (a<b)
	? kgV2_1(b, a, b%a, b, b%a)
	: kgV2_1(a, b, a%b, a, a%b) );
}

/**
 * Hilfsfunktion von kgV2.
 */
static int kgV2_1(int a, int b, int x0, int k, int x) {
  loops++;
  // x+x0 wird im echten "applikativen Algorithmus" natuerlich
  // nicht mit y abgekuerzt.
  int y = x + x0;
  if (x == 0) return k;
  else
  return kgV2_1(a, b, x0, k+a, (y >= b) ? (y - b) : y);
}

/**
 * die imperative Variante von kgV2.
 *
 * Die Schleife enthaelt kein Modulo, dafuer gegenueber
 * kgV0 einen zusaetzlichen Vergleich und zwei Additionen mehr.
 */
static int kgV3(int a, int b) {
  if (a < b) {int i = a; a = b; b = i;};
  int k = a, x0 = a%b, y; int x = x0;
  loops = 1;  // die Variablen-Deklaration..
  // beinhaltet schon den ersten Rechenschritt
  while (x != 0) {
	loops++;
	y = x + x0;
	x = (y >= b) ? (y - b) : y;
	k += a;
  };
  return(k);
}

public static void main(String args[]) {
  System.out.println("AlgDs WS99 Aufgabe 11, Christian Semrau 04.11.1999");
  int a, b, r;

  System.out.println("Berechnet das kleinste gemeinsame Vielfache.");
  do{
	System.out.print("Erste Zahl: "); a = IOUtils.readInt();
	System.out.print("Zweite Zahl:"); b = IOUtils.readInt();
	if ((a <= 0) || (b <= 0)) 
	  System.out.println("Das kgV kann nur von Zahlen >0 bestimmt werden!");
  }while((a <= 0) || (b <= 0));

  System.out.println("kgV1("+a+","+b+")="+ kgV1(a,b) +", "+loops);

} // main()
} // class ChS_Aufg11